<?php
defined('ABSPATH') or die('HEY, WHAT ARE YOU DOING HERE? YOU SILLY HUMAN!');
if (file_exists(ABSPATH.'wp-admin/includes/plugin.php')) {
    require_once(ABSPATH.'wp-admin/includes/plugin.php');
}
if ( ! class_exists('DBM_Plugin_Update')) {
    class DBM_Plugin_Update {

        private $plugin_data;
        private $plugin_name;
        private $plugin_slug;
        private $plugin_version;
        private $store_url;
        private $home_url;
        private $license_key;
        private $item_name;
        private $wp_override;
        private $timeout = 15;
        private $transient_key = '';

        public function __construct($_plugin_file, $args = null) {
            $this->plugin_data = get_plugin_data($_plugin_file);
            $this->plugin_name = plugin_basename($_plugin_file);
            $this->plugin_slug = wp_basename($_plugin_file, '.php');
            $this->plugin_version = $this->plugin_data['Version'];
            $this->store_url = isset($args['store_url']) ? trim($args['store_url']) : null;
            $this->home_url = home_url();
            $this->license_key = isset($args['license']) ? trim($args['license']) : null;
            $this->item_name = isset($args['item_name']) ? trim($args['item_name']) : null;
            $this->wp_override = isset($args['wp_override']) ? (bool)$args['wp_override'] : false;
            if (is_null($this->store_url) || is_null($this->item_name)) {
                return;
            }
            if ( ! filter_var($this->store_url, FILTER_VALIDATE_URL)) {
                return;
            }
            if ($this->store_url == $this->home_url) {
                return;
            }
            $this->transient_key = $this->plugin_slug.'_'.md5(serialize($this->plugin_slug.$this->store_url));
            $this->register_hooks();
        }

        public function register_hooks() {
            add_filter('pre_set_site_transient_update_plugins', [$this, 'check_plugin_updates']);
            add_filter('plugins_api', [$this, 'check_plugin_information'], 10, 3);
            if ( ! is_null($this->license_key)) {
                add_action("in_plugin_update_message-$this->plugin_name", [$this, 'check_licensed_plugin_status'], 20, 2);
            }
        }

        public function check_plugin_updates($_transient_data) {
            global $pagenow;
            if ( ! is_object($_transient_data)) {
                $_transient_data = new stdClass;
            }
            if ('plugins.php' == $pagenow && is_multisite()) {
                return $_transient_data;
            }
            if ( ! empty($_transient_data->response) && ! empty($_transient_data->response[$this->plugin_name]) && false === $this->wp_override) {
                return $_transient_data;
            }
            $cached_plugin_info = $this->get_cached_plugin_info();
            if (false === $cached_plugin_info) {
                $api_response = $this->api_request('plugin_latest_version');
                $this->set_plugin_info_cache($api_response);
            }
            if (false !== $cached_plugin_info && version_compare($this->plugin_version, $cached_plugin_info->new_version, '<')) {
                $_transient_data->last_checked = time();
                $_transient_data->response[$this->plugin_name] = $cached_plugin_info;
                $_transient_data->response[$this->plugin_name]->plugin = $this->plugin_name;
            }

            return $_transient_data;
        }

        public function check_plugin_information($_data, $_action = '', $_args = null) {
            if ($_action != 'plugin_information') {
                return $_data;
            }
            if (empty($this->plugin_slug)) {
                return $_data;
            }
            if ( ! isset($_args->slug) || ($_args->slug != $this->plugin_slug)) {
                return $_data;
            }
            $cached_plugin_info = $this->get_cached_plugin_info();
            if (false === $cached_plugin_info) {
                $api_response = $this->api_request('plugin_information');
                $this->set_plugin_info_cache($api_response);
                $cached_plugin_info = $api_response;
            }
            if (false !== $cached_plugin_info) {
                $_data = $cached_plugin_info;
            }
            if (isset($_data->sections) && ! is_array($_data->sections)) {
                $_data->sections = $this->convert_object_to_array($_data->sections);
            }
            if (isset($_data->banners) && ! is_array($_data->banners)) {
                $_data->banners = $this->convert_object_to_array($_data->banners);
            }
            if (isset($_data->icons) && ! is_array($_data->icons)) {
                $_data->icons = $this->convert_object_to_array($_data->icons);
            }
            if (isset($_data->contributors) && ! is_array($_data->contributors)) {
                $_data->contributors = $this->convert_object_to_array($_data->contributors);
            }
            if ( ! isset($_data->plugin)) {
                $_data->plugin = $this->plugin_name;
            }

            return $_data;
        }

        private function api_request($_action) {
            $api_params = [
                'edd_action' => 'get_download_updates',
                'license' => $this->license_key,
                'item_name' => $this->item_name,
                'version' => $this->plugin_version,
                'slug' => $this->plugin_slug,
                'url' => $this->home_url,
            ];
            $request = wp_remote_post($this->store_url, ['timeout' => $this->timeout, 'sslverify' => $this->verify_ssl(), 'body' => $api_params]);
            if ( ! is_wp_error($request)) {
                $request = json_decode(wp_remote_retrieve_body($request));
            }
            if ($request && isset($request->sections)) {
                $request->sections = maybe_unserialize($request->sections);
            } else {
                $request = false;
            }
            if ($request && isset($request->banners)) {
                $request->banners = maybe_unserialize($request->banners);
            }
            if ($request && isset($request->icons)) {
                $request->icons = maybe_unserialize($request->icons);
            }
            if ( ! empty($request->sections)) {
                foreach ($request->sections as $key => $section) {
                    $request->$key = (array)$section;
                }
            }

            return $request;
        }

        public function get_cached_plugin_info() {
            $cached_data = get_transient($this->transient_key);
            if ( ! empty($cached_data)) {
                return $cached_data;
            }

            return false;
        }

        public function set_plugin_info_cache($api_response) {
            set_transient($this->transient_key, $api_response, 60 * MINUTE_IN_SECONDS);
        }

        public function delete_cached_plugin_info() {
            delete_transient($this->transient_key);
        }

        public function check_licensed_plugin_status() {
            $data = $this->get_cached_plugin_info();
            if (isset($data->license_status) && ! empty($data->license_status)) {
                echo $data->license_status;
            }

            return null;
        }

        private function verify_ssl() {
            return (bool)apply_filters('dc_edd_sl_api_request_verify_ssl', true, $this);
        }

        private function convert_object_to_array($data) {
            $new_data = [];
            foreach ($data as $key => $value) {
                $new_data[$key] = is_object($value) ? $this->convert_object_to_array($value) : $value;
            }

            return $new_data;
        }

    }
}