<?php

class DCSBCM_Divi_Breadcrumbs_Module extends ET_Builder_Module {

    public $slug = 'et_pb_dcsbcm_divi_breadcrumbs_module';

    public $vb_support = 'on';

    public $main_css_element = '%%order_class%%';

    protected $module_credits = [
        'module_uri' => 'https://divibreadcrumbs.com/',
        'author' => 'Divi Codex',
        'author_uri' => 'https://divicake.com/author/divicodex/',
    ];

    public function init() {
        $this->name = esc_html__('Divi Breadcrumbs', 'dcsbcm_divi_breadcrumbs_module');
        $this->main_css_element = '%%order_class%%';
        $this->settings_modal_toggles = [
            'general' => [
                'toggles' => [
                    'breadcrumbnav_settings' => esc_html__('Breadcrumb Navigation Settings', 'dcsbcm_divi_breadcrumbs_module'),
                ],
            ],
        ];
        $this->custom_css_fields = [
            'breadcrumbcss_breadcrumbs' => [
                'label' => esc_html__('Breadcrumbs', 'dcsbcm_divi_breadcrumbs_module'),
                'selector' => 'span.dcsbcm_divi_breadcrumb',
            ],
            'breadcrumbcss_activebreadcrumb' => [
                'label' => esc_html__('Active Breadcrumb', 'dcsbcm_divi_breadcrumbs_module'),
                'selector' => 'span.dcsbcm_divi_breadcrumb-active',
            ],
            'breadcrumbcss_breadcrumblinks' => [
                'label' => esc_html__('Breadcrumb Links', 'dcsbcm_divi_breadcrumbs_module'),
                'selector' => 'span.dcsbcm_divi_breadcrumb a',
            ],
            'breadcrumbcss_breadcrumblinkhover' => [
                'label' => esc_html__('Breadcrumb Link Hover', 'dcsbcm_divi_breadcrumbs_module'),
                'selector' => 'span.dcsbcm_divi_breadcrumb a:hover',
            ],
            'breadcrumbcss_separators' => [
                'label' => esc_html__('Separators', 'dcsbcm_divi_breadcrumbs_module'),
                'selector' => 'span.dcsbcm_separator',
            ],
        ];
    }

    public function get_fields() {
        // Init separator choices
        $dcsbcm_separatorlist = [];
        $dcsbcm_separatorlist['sep-raquo'] = '»';
        $dcsbcm_separatorlist['sep-arrow'] = '→';
        $dcsbcm_separatorlist['sep-tri'] = '▶';
        $dcsbcm_separatorlist['sep-dash'] = '-';
        $dcsbcm_separatorlist['sep-ndash'] = '–';
        $dcsbcm_separatorlist['sep-mdash'] = '—';
        $dcsbcm_separatorlist['sep-tinydot'] = '·';
        $dcsbcm_separatorlist['sep-bull'] = '•';
        $dcsbcm_separatorlist['sep-tinystar'] = '*';
        $dcsbcm_separatorlist['sep-star'] = '⋆';
        $dcsbcm_separatorlist['sep-tilde'] = '~';
        $dcsbcm_separatorlist['sep-pipe'] = '|';
        $module_fields = [];
        $module_fields['hide_homebreadcrumb'] = [
            'label' => esc_html__('Hide Home Breadcrumb', 'dcsbcm_divi_breadcrumbs_module'),
            'type' => 'yes_no_button',
            'option_category' => 'basic_option',
            'default' => 'off',
            'options' => [
                'off' => esc_html__("No", 'dcsbcm_divi_breadcrumbs_module'),
                'on' => esc_html__('Yes', 'dcsbcm_divi_breadcrumbs_module'),
            ],
            'affects' => [
                'homebreadcrumbtext'
            ],
            'toggle_slug' => 'breadcrumbnav_settings',
            'description' => esc_html__('Choose whether or not to display "Home" link.', 'dcsbcm_divi_breadcrumbs_module'),
            'computed_affects' => [
                'computed_field_html_payload',
            ],
        ];
        $module_fields['homebreadcrumbtext'] = [
            'label' => esc_html__('Home Breadcrumb Text', 'dcsbcm_divi_breadcrumbs_module'),
            'type' => 'text',
            'option_category' => 'basic_option',
            'depends_show_if' => 'off',
            'description' => esc_html__('If you would like override the "Home" breadcrumb text, input your own text here. The word "Home" will be shown if this field is left blank.',
                'dcsbcm_divi_breadcrumbs_module'),
            'toggle_slug' => 'breadcrumbnav_settings',
            'computed_affects' => [
                'computed_field_html_payload',
            ],
        ];
        $module_fields['separator'] = [
            'label' => esc_html__('Separator', 'dcsbcm_divi_breadcrumbs_module'),
            'type' => 'select',
            'option_category' => 'basic_option',
            'default' => 'sep-raquo',
            'options' => $dcsbcm_separatorlist,
            'toggle_slug' => 'breadcrumbnav_settings',
            'description' => esc_html__('Choose a symbol to use as your breadcrumb separator. The appearance of separators can be customized in the Design Tab.', 'dcsbcm_divi_breadcrumbs_module'),
            'computed_affects' => [
                'computed_field_html_payload',
            ],
        ];
        $module_fields['hide_currentbreadcrumb'] = [
            'label' => esc_html__('Hide Current Page', 'dcsbcm_divi_breadcrumbs_module'),
            'type' => 'yes_no_button',
            'option_category' => 'basic_option',
            'options' => [
                'off' => esc_html__("No", 'dcsbcm_divi_breadcrumbs_module'),
                'on' => esc_html__('Yes', 'dcsbcm_divi_breadcrumbs_module'),
            ],
            'toggle_slug' => 'breadcrumbnav_settings',
            'description' => esc_html__('Choose whether or not to display the show current Post or Page title in the breadcrumbs.', 'dcsbcm_divi_breadcrumbs_module'),
            'computed_affects' => [
                'computed_field_html_payload',
            ],
        ];
        $module_fields['category_breadcrumb_text'] = [
            'label' => esc_html__('Category Breadcrumb Text', 'dcsbcm_divi_breadcrumbs_module'),
            'type' => 'text',
            'option_category' => 'basic_option',
            'depends_show_if' => 'off',
            'description' => esc_html__('If you would like to override the "Category" breadcrumb text, input your own text here. The word "Category" will be shown on category pages if this field is left blank.',
                'dcsbcm_divi_breadcrumbs_module'),
            'toggle_slug' => 'breadcrumbnav_settings',
            'computed_affects' => [
                'computed_field_html_payload',
            ],
        ];
        $module_fields['computed_field_html_payload'] = [
            'type' => 'computed',
            'computed_callback' => ['dcsbcm_Divi_Breadcrumbs_Module', 'dcsbcm_build_output'],
            'computed_depends_on' => [
                'hide_homebreadcrumb',
                'homebreadcrumbtext',
                'separator',
                'hide_currentbreadcrumb',
            ],
            'computed_minimum' => [
                'hide_homebreadcrumb',
                'homebreadcrumbtext',
                'separator',
                'hide_currentbreadcrumb',
            ],
        ];

        return $module_fields;
    }

    public function get_advanced_fields_config() {
        $advanced_fields = [];
        $advanced_fields['link_options'] = false;
        $advanced_fields['text'] = false;
        $advanced_fields['fonts'] = [
            'body' => [
                'label' => esc_html__('Breadcrumbs', 'dcsbcm_divi_breadcrumbs_module'),
                'css' => [
                    'main' => "{$this->main_css_element}",
                    'important' => 'all',
                ],
            ],
            'fontsseparator' => [
                'label' => esc_html__('Separator', 'dcsbcm_divi_breadcrumbs_module'),
                'css' => [
                    'main' => "{$this->main_css_element} span.dcsbcm_separator",
                    'important' => 'all',
                ],
            ],
            'fontsbreadcrumblinks' => [
                'label' => esc_html__('Breadcrumb Links', 'dcsbcm_divi_breadcrumbs_module'),
                'css' => [
                    'main' => "{$this->main_css_element} li.dcsbcm_divi_breadcrumb a",
                ],
            ],
        ];
        $advanced_fields['custom_margin_padding'] = [
            'css' => [
                'important' => 'all',
            ],
        ];

        return $advanced_fields;
    }

    public static function dcsbcm_build_output($args = [], $conditional_tags = [], $current_page = []) {
        $args = wp_parse_args($args);
        ob_start();
        // Retrieve of all the stored data into variables to be used in the method.
        $separator = $args['separator'];
        $hide_homebreadcrumb = $args['hide_homebreadcrumb'];
        $homebreadcrumbtext = $args['homebreadcrumbtext'];
        $hide_currentbreadcrumb = $args['hide_currentbreadcrumb'];
        $category_breadcrumb_text = isset($args['category_breadcrumb_text']) ? $args['category_breadcrumb_text'] : '';
        $current_page_ID = $current_page['id'];
        $current_page_slug = get_post_field('post_name', $current_page_ID);
        $post = get_post($current_page_ID);
        // Configure Home breadcrumb text
        $breadcrumb_full_text['home'] = $homebreadcrumbtext;
        if ($homebreadcrumbtext == '') {
            $breadcrumb_full_text['home'] = 'Home';
        } // If value is blank, set to "Home".
        // Show or don't show Home breadcrumb
        if ($hide_homebreadcrumb == 'on') {
            $hide_homebreadcrumb = 1; // 0 - don't hide home title in breadcrumbs, 1 - hide
        } else {
            $hide_homebreadcrumb = 0; // 0 - don't hide home title in breadcrumbs, 1 - hide
        }
        // Show or don't show Current page breadcrumb
        if ($hide_currentbreadcrumb == 'on') {
            $hide_currentbreadcrumb = 1; // 0 - don't hide current post/page title in breadcrumbs, 1 - hide
        } else {
            $hide_currentbreadcrumb = 0; // 0 - don't hide current post/page title in breadcrumbs, 1 - hide
        }
        // Separator Options
        switch ($separator) {
            case 'sep-raquo':
                $separator = '&raquo;';
                break;
            case 'sep-arrow':
                $separator = '&rarr;';
                break;
            case 'sep-tri':
                $separator = '&#9654;';
                break;
            case 'sep-dash':
                $separator = '-';
                break;
            case 'sep-ndash':
                $separator = '&ndash;';
                break;
            case 'sep-mdash':
                $separator = '&mdash;';
                break;
            case 'sep-tinydot':
                $separator = '·';
                break;
            case 'sep-bull':
                $separator = '&bull;';
                break;
            case 'sep-tinystar':
                $separator = '*';
                break;
            case 'sep-star':
                $separator = '&#9733;';
                break;
            case 'sep-tilde':
                $separator = '~';
                break;
            case 'sep-pipe':
                $separator = '|';
                break;
        }
        $separator = '<span class="dcsbcm_separator">&nbsp;'.$separator.'&nbsp;</span>';
        $breadcrumb_active_before = '<span class="dcsbcm_divi_breadcrumb dcsbcm_divi_breadcrumb-active">'; // tag before the active crumb
        $breadcrumb_active_after = '</span>';                                                              // tag after the active crumb
        $cat_text = ' "%s" ';
        if ($category_breadcrumb_text == '') {
            $breadcrumb_full_text['category'] = "Category: $cat_text"; // text for a category page
        } else {
            $breadcrumb_full_text['category'] = "$category_breadcrumb_text:$cat_text"; // text for a category page
        }
        $breadcrumb_full_text['tax'] = 'Archive for "%s"';           // text for a taxonomy page
        $breadcrumb_full_text['search'] = 'Search Results for "%s"'; // text for a search results page
        $breadcrumb_full_text['tag'] = 'Posts Tagged "%s"';          // text for a tag page
        $breadcrumb_full_text['author'] = 'Posts by %s';             // text for an author page
        $breadcrumb_full_text['404'] = 'Error 404';                  // text for the 404 page
        $homeLink = get_bloginfo('url');
        $position = 1;
        $link_before = '<li class="dcsbcm_divi_breadcrumb" property="itemListElement" typeof="ListItem">';
        $link_after = '<meta property="position" content="1"></li>';
        $link_attr = ' property="item" typeof="WebPage"';
        $link = $link_before.'<a'.$link_attr.' href="%1$s">%2$s</a>'.$link_after;
        $dcsbcm_output_breadcrumbshtml = '<ol class="dcsbcm_divi_breadcrumbs" vocab="http://schema.org/" typeof="BreadcrumbList">';
        if (is_home() || is_front_page()) {
            if ($hide_homebreadcrumb == 0) {
                /*$dcsbcm_output_breadcrumbshtml = '<div class="dcsbcm_divi_breadcrumbs"><span class="dcsbcm_divi_breadcrumb dcsbcm_divi_breadcrumb-active"><a href="'.$homeLink.'">'
                                                 .$breadcrumb_full_text['home'].'</a></span></div>';*/
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, $homeLink, $breadcrumb_full_text['home'], 1);
            }
        } else {
            if ($hide_homebreadcrumb == 0) {
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, $homeLink, $breadcrumb_full_text['home']).$separator;
            }
            if (is_category()) {
                $thisCat = get_category(get_query_var('cat'), false);
                if ($thisCat->parent != 0) {
                    $cats = get_category_parents($thisCat->parent, true, $separator);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('</a>', '</a>'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['category'], single_cat_title('', false)).$breadcrumb_active_after;
            } elseif (is_tax()) {
                $thisCat = get_category(get_query_var('cat'), false);
                if ($thisCat->parent != 0) {
                    $cats = get_category_parents($thisCat->parent, true, $separator);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('</a>', '</a>'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['tax'], single_cat_title('', false)).$breadcrumb_active_after;
            } elseif (is_search()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['search'], get_search_query()).$breadcrumb_active_after;
            } elseif (is_day()) {
                $position = $position + 1;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_year_link(get_the_time('Y')), get_the_time('Y'), $position).$separator;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_month_link(get_the_time('Y'), get_the_time('m')), get_the_time('F'), $position).$separator;
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('d').$breadcrumb_active_after;
            } elseif (is_month()) {
                $position = $position + 1;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_year_link(get_the_time('Y')), get_the_time('Y'), $position).$separator;
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('F').$breadcrumb_active_after;
            } elseif (is_year()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('Y').$breadcrumb_active_after;
            } elseif (is_single($current_page_ID) && ! is_attachment()) {
                if (get_post_type($current_page_ID) != 'post') {
                    $post_type = get_post_type_object(get_post_type($current_page_ID));
                    $slug = $post_type->rewrite;
                    printf($link, $homeLink.'/'.$slug['slug'].'/', $post_type->labels->singular_name);
                    if ($hide_currentbreadcrumb == 0) {
                        $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                    }
                } else {
                    $cat = get_the_category();
                    $cat = $cat[0];
                    $cats = get_category_parents($cat, true, $separator);
                    if ($hide_currentbreadcrumb == 1) {
                        $cats = preg_replace("#^(.+)$separator$#", "$1", $cats);
                    }
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('</a>', '</a>'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                    if ($hide_currentbreadcrumb == 0) {
                        $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                    }
                }
            } elseif ( ! is_single($current_page_ID) && get_post_type($current_page_ID) !== 'page' && get_post_type($current_page_ID) != 'post' && ! is_404()) {
                $post_type = get_post_type_object(get_post_type($current_page_ID));
                if ( ! is_object($post_type) || ! is_a($post_type, 'WP_Post')) {
                    $post_type = $GLOBALS['wp_the_query']->queried_object;
                }
                $singular_name = '';
                if (isset($post_type->labels->singular_name)) {
                    $singular_name = $post_type->labels->singular_name;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.$singular_name.$breadcrumb_active_after;
            } elseif (is_attachment()) {
                $parent = get_post($post->post_parent);
                $cat = get_the_category($parent->ID);
                $cat = $cat[0];
                $cats = get_category_parents($cat, true, $separator);
                $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                $cats = str_replace('</a>', '</a>'.$link_after, $cats);
                $dcsbcm_output_breadcrumbshtml .= $cats;
                printf($link, get_permalink($parent), $parent->post_title);
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (get_post_type($current_page_ID) == 'page' && ! $post->post_parent) {
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (get_post_type($current_page_ID) == 'page' && $post->post_parent) {
                $parent_id = $post->post_parent;
                $breadcrumbs = [];
                while ($parent_id) {
                    $page = get_post($parent_id);
                    $breadcrumbs[] = sprintf($link, get_permalink($page->ID), get_the_title($page->ID));
                    $parent_id = $page->post_parent;
                }
                $breadcrumbs = array_reverse($breadcrumbs);
                for ($i = 0; $i < count($breadcrumbs); $i++) {
                    $dcsbcm_output_breadcrumbshtml .= $breadcrumbs[$i];
                    if ($i != count($breadcrumbs) - 1) {
                        $dcsbcm_output_breadcrumbshtml .= $separator;
                    }
                }
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (is_tag()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['tag'], single_tag_title('', false)).$breadcrumb_active_after;
            } elseif (is_author()) {
                global $author;
                $userdata = get_userdata($author);
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['author'], $userdata->display_name).$breadcrumb_active_after;
            } elseif (is_404()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.$breadcrumb_full_text['404'].$breadcrumb_active_after;
            }
            if (get_query_var('paged')) {
                if (is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author()) {
                    $dcsbcm_output_breadcrumbshtml .= ' (';
                }
                $dcsbcm_output_breadcrumbshtml .= __('Page').' '.get_query_var('paged');
                if (is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author()) {
                    $dcsbcm_output_breadcrumbshtml .= ')';
                }
            }
        }
        $dcsbcm_output_breadcrumbshtml .= '</ol>';
        if (is_home() || is_front_page()) {
            if ($hide_homebreadcrumb == 0) {
                echo $dcsbcm_output_breadcrumbshtml;
            }
        } else {
            echo $dcsbcm_output_breadcrumbshtml;
        }

        return ob_get_clean();
    }

    public function render($attrs, $content = null, $render_slug) {
        // Retrieve of all the stored data into variables to be used in the method.
        $separator = $this->props['separator'];
        $hide_homebreadcrumb = $this->props['hide_homebreadcrumb'];
        $homebreadcrumbtext = $this->props['homebreadcrumbtext'];
        $hide_currentbreadcrumb = $this->props['hide_currentbreadcrumb'];
        $category_breadcrumb_text = $this->props['category_breadcrumb_text'];
        $current_page_ID = get_the_ID();
        $current_page_slug = get_post_field('post_name', $current_page_ID);
        $post = get_post($current_page_ID);
        // Configure Home breadcrumb text
        $breadcrumb_full_text['home'] = $homebreadcrumbtext;
        if ($homebreadcrumbtext == '') {
            $breadcrumb_full_text['home'] = 'Home';
        } // If value is blank, set to "Home".
        // Show or don't show Home breadcrumb
        if ($hide_homebreadcrumb == 'on') {
            $hide_homebreadcrumb = 1; // 0 - don't hide home title in breadcrumbs, 1 - hide
        } else {
            $hide_homebreadcrumb = 0; // 0 - don't hide home title in breadcrumbs, 1 - hide
        }
        // Show or don't show Current page breadcrumb
        if ($hide_currentbreadcrumb == 'on') {
            $hide_currentbreadcrumb = 1; // 0 - don't hide current post/page title in breadcrumbs, 1 - hide
        } else {
            $hide_currentbreadcrumb = 0; // 0 - don't hide current post/page title in breadcrumbs, 1 - hide
        }
        // Separator Options
        switch ($separator) {
            case 'sep-raquo':
                $separator = '&raquo;';
                break;
            case 'sep-arrow':
                $separator = '&rarr;';
                break;
            case 'sep-tri':
                $separator = '&#x25BA;';
                break;
            case 'sep-dash':
                $separator = '-';
                break;
            case 'sep-ndash':
                $separator = '&ndash;';
                break;
            case 'sep-mdash':
                $separator = '&mdash;';
                break;
            case 'sep-tinydot':
                $separator = '·';
                break;
            case 'sep-bull':
                $separator = '&bull;';
                break;
            case 'sep-tinystar':
                $separator = '*';
                break;
            case 'sep-star':
                $separator = '&#9733;';
                break;
            case 'sep-tilde':
                $separator = '~';
                break;
            case 'sep-pipe':
                $separator = '|';
                break;
        }
        $separator = '<span class="dcsbcm_separator">&nbsp;'.$separator.'&nbsp;</span>';
        $breadcrumb_active_before = '<span class="dcsbcm_divi_breadcrumb dcsbcm_divi_breadcrumb-active">'; // tag before the active crumb
        $breadcrumb_active_after = '</span>';                                                              // tag after the active crumb
        $cat_text = ' "%s" ';
        if ($category_breadcrumb_text == '') {
            $breadcrumb_full_text['category'] = "Category: $cat_text"; // text for a category page
        } else {
            $breadcrumb_full_text['category'] = "$category_breadcrumb_text:$cat_text"; // text for a category page
        }
        $breadcrumb_full_text['tax'] = 'Archive for "%s"';           // text for a taxonomy page
        $breadcrumb_full_text['search'] = 'Search Results for "%s"'; // text for a search results page
        $breadcrumb_full_text['tag'] = 'Posts Tagged "%s"';          // text for a tag page
        $breadcrumb_full_text['author'] = 'Posts by %s';             // text for an author page
        $breadcrumb_full_text['404'] = 'Error 404';                  // text for the 404 page
        $homeLink = get_bloginfo('url');
        $link_before = '<li class="dcsbcm_divi_breadcrumb" property="itemListElement" typeof="ListItem">';
        $position = 1;
        $link_after = '</li>';
        $link_attr = ' property="item" typeof="WebPage"';
        $link = $link_before.'<a'.$link_attr.' href="%1$s"><span property="name">%2$s</span></a><meta property="position" content="%3$s">'.$link_after;
        $position = 1;
        $dcsbcm_output_breadcrumbshtml = '<ol class="dcsbcm_divi_breadcrumbs" vocab="http://schema.org/" typeof="BreadcrumbList">';
        if (is_home() || is_front_page()) {
            if ($hide_homebreadcrumb == 0) {
                /*$dcsbcm_output_breadcrumbshtml = '<div class="dcsbcm_divi_breadcrumbs"><span class="dcsbcm_divi_breadcrumb dcsbcm_divi_breadcrumb-active"><a href="'.$homeLink.'">'
                .$breadcrumb_full_text['home'].'</a></span></div>'*/;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, $homeLink, $breadcrumb_full_text['home'], 1);
            }
        } else {
            if ($hide_homebreadcrumb == 0) {
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, $homeLink, $breadcrumb_full_text['home'], $position).$separator;
            }
            if (is_category()) {
                $thisCat = get_category(get_query_var('cat'), false);
                if ($thisCat->parent != 0) {
                    $cats = get_category_parents($thisCat->parent, true, $separator);
                    $cats = preg_replace('/(<a\b[^><]*)>/i', '$1 toAddPropertyName>', $cats);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('toAddPropertyName>', '><span property="name">', $cats);
                    $position = $position + 1;
                    $cats = str_replace('</a>', '</span></a><meta property="position" content="'.$position.'">'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['category'], single_cat_title('', false)).$breadcrumb_active_after;
            } elseif (is_tax()) {
                $thisCat = get_category(get_query_var('cat'), false);
                if (isset($category->parent)) {
                    $cats = get_category_parents($thisCat->parent, true, $separator);
                    $cats = preg_replace('/(<a\b[^><]*)>/i', '$1 toAddPropertyName>', $cats);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('toAddPropertyName>', '><span property="name">', $cats);
                    $position = $position + 1;
                    $cats = str_replace('</a>', '</span></a><meta property="position" content="'.$position.'">'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['tax'], single_cat_title('', false)).$breadcrumb_active_after;
            } elseif (is_search()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['search'], get_search_query()).$breadcrumb_active_after;
            } elseif (is_day()) {
                $position = $position + 1;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_year_link(get_the_time('Y')), get_the_time('Y'), $position).$separator;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_month_link(get_the_time('Y'), get_the_time('m')), get_the_time('F')).$separator;
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('d').$breadcrumb_active_after;
            } elseif (is_month()) {
                $position = $position + 1;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_year_link(get_the_time('Y')), get_the_time('Y'), $position).$separator;
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('F').$breadcrumb_active_after;
            } elseif (is_year()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_time('Y').$breadcrumb_active_after;
            } elseif (is_single($current_page_ID) && ! is_attachment()) {
                if (get_post_type($current_page_ID) != 'post') {
                    $post_type = get_post_type_object(get_post_type($current_page_ID));
                    $slug = $post_type->rewrite;
                    $position = $position + 1;
                    $dcsbcm_output_breadcrumbshtml .= sprintf($link, $homeLink.'/'.$slug['slug'].'/', $post_type->labels->singular_name, $position);
                    if ($hide_currentbreadcrumb == 0) {
                        $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                    }
                } else {
                    $cat = get_the_category();
                    $cat = $cat[0];
                    $position = $position + 1;
                    $cats = get_category_parents($cat, true, $separator);
                    if ($hide_currentbreadcrumb == 1) {
                        $cats = preg_replace("#^(.+)$separator$#", "$1", $cats);
                    }
                    $cats = preg_replace('/(<a\b[^><]*)>/i', '$1 toAddPropertyName>', $cats);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('toAddPropertyName>', '><span property="name">', $cats);
                    $cats = str_replace('</a>', '</span></a><meta property="position" content="'.$position.'">'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                    if ($hide_currentbreadcrumb == 0) {
                        $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                    }
                }
            } elseif ( ! is_single($current_page_ID) && get_post_type($current_page_ID) !== 'page' && get_post_type($current_page_ID) != 'post' && ! is_404()) {
                $post_type = get_post_type_object(get_post_type($current_page_ID));
                if ( ! is_object($post_type) || ! is_a($post_type, 'WP_Post')) {
                    $post_type = $GLOBALS['wp_the_query']->queried_object;
                }
                $singular_name = '';
                if (isset($post_type->labels->singular_name)) {
                    $singular_name = $post_type->labels->singular_name;
                }
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.$singular_name.$breadcrumb_active_after;
            } elseif (is_attachment()) {
                $parent = get_post($post->post_parent);
                $cat = get_the_category($parent->ID);
                if ( ! empty($cat)) {
                    $cat = $cat[0];
                    $cats = get_category_parents($cat, true, $separator);
                    $cats = str_replace('<a', $link_before.'<a'.$link_attr, $cats);
                    $cats = str_replace('</a>', '</a>'.$link_after, $cats);
                    $dcsbcm_output_breadcrumbshtml .= $cats;
                }
                $position = $position + 1;
                $dcsbcm_output_breadcrumbshtml .= sprintf($link, get_permalink($parent), $parent->post_title, $position);
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (get_post_type($current_page_ID) == 'page' && ! $post->post_parent) {
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (get_post_type($current_page_ID) == 'page' && $post->post_parent) {
                $parent_id = $post->post_parent;
                $breadcrumbs = [];
                while ($parent_id) {
                    $position = $position + 1;
                    $page = get_post($parent_id);
                    $breadcrumbs[] = sprintf($link, get_permalink($page->ID), get_the_title($page->ID), $position);
                    $parent_id = $page->post_parent;
                }
                $breadcrumbs = array_reverse($breadcrumbs);
                for ($i = 0; $i < count($breadcrumbs); $i++) {
                    $dcsbcm_output_breadcrumbshtml .= $breadcrumbs[$i];
                    if ($i != count($breadcrumbs) - 1) {
                        $dcsbcm_output_breadcrumbshtml .= $separator;
                    }
                }
                if ($hide_currentbreadcrumb == 0) {
                    $dcsbcm_output_breadcrumbshtml .= $separator.$breadcrumb_active_before.get_the_title($current_page_ID).$breadcrumb_active_after;
                }
            } elseif (is_tag()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['tag'], single_tag_title('', false)).$breadcrumb_active_after;
            } elseif (is_author()) {
                global $author;
                $userdata = get_userdata($author);
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.sprintf($breadcrumb_full_text['author'], $userdata->display_name).$breadcrumb_active_after;
            } elseif (is_404()) {
                $dcsbcm_output_breadcrumbshtml .= $breadcrumb_active_before.$breadcrumb_full_text['404'].$breadcrumb_active_after;
            }
            if (get_query_var('paged')) {
                if (is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author()) {
                    $dcsbcm_output_breadcrumbshtml .= ' (';
                }
                $dcsbcm_output_breadcrumbshtml .= __('Page').' '.get_query_var('paged');
                if (is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author()) {
                    $dcsbcm_output_breadcrumbshtml .= ')';
                }
            }
        }
        $dcsbcm_output_breadcrumbshtml .= '</ol>';
        if (is_home() || is_front_page()) {
            if ($hide_homebreadcrumb == 0) {
                return $dcsbcm_output_breadcrumbshtml;
            } // If on the homepage, only return html if we're supposed to.
        } else {
            return $dcsbcm_output_breadcrumbshtml;
        }
    }

}

(new DCSBCM_Divi_Breadcrumbs_Module);